% Q:\trunk\reiner\ShannonKriging\TestResamplingMethods\PlotMDPI_Mar30\Plot_MDPI_Mrz30.m
% (c) Reiner Jedermann Univesity Bremen, http://intelligentcontainer.com
% This code might be freely used, if the main article is cited:
% Jedermann, R.: Feasibility of Low Latency, Single-Sample Delay Resampling: A New Kriging Based Method. In: Algorithms 16, no. 4: 203. (doi:  10.3390/a16040203)

clear
figNum = 130;  % Mulitply figure number with 10 and add 1 or 2, for figure (a), (b)
               % 60, 91, 92, 100, 111, 112, 120, 130

RMSE_THRESHOLD = -40;  % set to -40 dB (default) or - 60 dB

LEGEND_OSR = 1;
fileName = 'SimResults_Mar30.json'; 

RMSE_ALL = 0;
RMSE_EDGE = 0;
RMSE_CENTER = 1;
COMPARE_MODE  = 0;

fontSize = 14;
legensSize = fontSize-1;

colorMatrix = colororder;
cUni  = colorMatrix(5,:);
cJit1 = colorMatrix(1,:);
cJit2r = colorMatrix(2,:);
cMis1y = colorMatrix(3,:);
cMis2 = colorMatrix(4,:);
cHelp = colorMatrix(6,:);

fid = fopen(fileName); % Opening the file
raw = fread(fid,inf); % Reading the contents
str = char(raw'); % Transformation
fclose(fid); % Closing the file
data = jsondecode(str); % Using the jsondecode function to parse JSON from string

figure(figNum)
set(gca,'LineWidth',0.7);
subplot('Position',[0.13 0.13 0.85 0.85])

colorMatrix = colororder;

testShort = sprintf('T%i', figNum);
oneTest = data.(testShort);
F_List = oneTest.F_List;

methods = oneTest.Methods;
m_short = fieldnames(methods);
n_methods = length(m_short);

noiseLevel = 0;
plotName = sprintf('T%i: %s', figNum, oneTest.Name);
plotName = strrep(plotName, '_', '\_');

plotName = [plotName ' : ' oneTest.Name];

PlotList = 1:n_methods;
plotTitle = plotName;
figNumTxt = '?';

shift_Y = 10;

% default values
colorSet = {cJit2r, cUni, cJit1, cMis1y, cMis2, cHelp };
lineWidths = [1 1 1 1 1 1 1 ];
markerList = '-d-d-d-d-d';          
        
osr_cell = cell(n_methods, 1);
f40_cell = cell(n_methods, 1);

method_cell = cell(n_methods, 1);
for i = 1:n_methods
    key = m_short{i};
    m_name = methods.(key);
    method_cell{i} = m_name; 
end
legend0 = method_cell;

methodSet4 =  [
    {'Kriging'}
    {'Margolis'}
    {'Lagrange'}
    {'Akima'}         
            ];
            
switch(figNum)

    case {60}  
        figNumTxt = 'Fig 6 compare Methods uniform'; 
        plotTitle = 'Uniform, group delay';
        RMSE_EDGE = 1;
        markerList = '------';
        legend0 = methodSet4;

    case {91}  
        figNumTxt = 'Fig 9a Group delay clean jitter/missing, WSI for uniform/missing'; 
        plotTitle = '(a) Group delay';
        COMPARE_MODE = 1;
        legensSize = fontSize-2;
        legend0 = [
            {'Kriging'}
            {'Margolis'}
            {'Lagrange'}
            {'Akima'}
            {'WSI'}         
            ];
          
    case {92}  
        figNumTxt = 'Fig 9b Group delay noise jitter/missing'; 
        plotTitle = '(b) Group delay, noise';
        COMPARE_MODE = 1;
        legend0 = methodSet4;
          
    case {100}  
        figNumTxt = 'Fig 10 Single delay clean '; 
        plotTitle = 'Single delay';
        COMPARE_MODE = 1;
        legend0 = methodSet4;

       
    case {111}  
        figNumTxt = 'Fig 11a Single delay quantization'; 
        plotTitle = '(a) Single delay, quantization';
        COMPARE_MODE = 1;
        legend0 = methodSet4;
        
     case {112}  
        figNumTxt = 'Fig 11b Single delay noise'; 
        plotTitle = '(b) Single delay, noise';
        COMPARE_MODE = 1;
        legend0 = methodSet4;
        
    
    
    case {120}
        colorSet = {cJit2r, cMis1y, cJit2r, cMis1y};
        figNumTxt = 'Fig 8 Nugget for Update, r=6.6'; 
        lineWidths = [2 2 1 1  ];
        markerList = '::--';
        plotTitle = {'Kriging, missing samples', 'single delay'};
        legend0 = [
            {'Quantization 12 bit, n{\fontsize{8}g}=0'}
            {'Noise -60 dB, n{\fontsize{8}g}=0'}
            {'Quantization 12 bit, n{\fontsize{8}g}=10^-^8'}
            {'Noise -60 dB, n{\fontsize{8}g}= 10^-^6'}
            ];
        
    case {130}
        figNumTxt = 'Fig 12 Future no Noise/Noise -60dB, n_g=1e-6';
        plotTitle = {'Kriging, missing samples', 'single delay'};
        if figNum==121
            plotTitle = 'Margolis future samples';
        end
        lineWidths = [1 2 1 1 1];
        markerList = '-d-d-d-d-d';  
        colorSet = {cHelp, cMis1y, cJit2r, cUni, cMis2};   
        COMPARE_MODE = 1;
        legend01 = [
            {'0 Future samples'}
            {'1 Future sample'}
            {'2 Future samples'}
            {'3 Future samples'}
            {'4 Future samples'}
            ];
        legend0 = [
            {'0 FS'}
            {'1 FS'}
            {'2 FS'}
            {'3 FS'}
            {'4 FS'}

            ];


        
        

end

theLegend = cell(length(legend0),1);
if iscell(plotTitle)
    fprintf('%s --> %s \n', plotName, plotTitle{1});
else
    fprintf('%s --> %s \n', plotName, plotTitle);
end

count = 0;
count_leg = 1;
oneLegend = legend0{1};


for i=PlotList
    count = count +1;
    key = m_short{i};
    
    lw = lineWidths(count_leg);
    color = colorSet{count_leg};   
    marker1 = markerList(count);
    if marker1 == 'd'
        marker1 = '--';
    end

    
    osr_List = '';
    f40_List = '';
    
    if RMSE_CENTER == 1
        rmse_AC = oneTest.RmseCenter.(key);
        fAC = findIntersection(F_List, rmse_AC, RMSE_THRESHOLD);   
        osr_List = sprintf('%s %s', osr_List, OSR_text(fAC));
        f40_List = sprintf('%s %s', f40_List, f40_text(fAC));
    end
    
    if RMSE_EDGE == 1
        rmse_E = oneTest.RmseEdges.(key);
        fE = findIntersection(F_List, rmse_E, RMSE_THRESHOLD);
        osr_List = sprintf('%s %s', osr_List, OSR_text(fE));
        f40_List = sprintf('%s %s', f40_List, f40_text(fAC));
    end
    osr_cell{i} = osr_List;
    f40_cell{i} = f40_List;
 

    
    if RMSE_CENTER == 1        
        hLine = plot(F_List, db(rmse_AC), marker1, 'Color', color, 'LineWidth', lw);
        hold on
        if COMPARE_MODE == 1
            if mod(count,2)==0 
                % fprintf('ignore Legend %i \n', count)
                set(get(get(hLine,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
                oneLegend = sprintf('%s / %s %s', lastOSR, OSR_text(fAC), oneLegend);
                theLegend{count_leg} = oneLegend;
                count_leg = count_leg + 1;
                if count_leg <= length(legend0)
                    oneLegend = legend0{count_leg};
                end
            end
        else
            oneLegend = sprintf('%s %s', OSR_text(fAC), oneLegend);
            theLegend{count_leg} = oneLegend;
            count_leg = count_leg + 1;
            if count_leg <= length(legend0)
                oneLegend = legend0{count_leg};
            end
        end
        lastOSR = OSR_text(fAC);
        fprintf('%s --> %s \n', key, oneLegend);
        
        if(fAC<0.5)
            hLine = plot(fAC, RMSE_THRESHOLD, 'd', 'Color', color);
            set(get(get(hLine,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
        end
        
    end
    
    if RMSE_EDGE == 1
        hLine = plot(F_List, db(rmse_E), ':', 'Color', color, 'LineWidth', lw);  %%%%%%%%%%%%%% --
        set(get(get(hLine,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
        hLine = plot(fE, RMSE_THRESHOLD, 'd', 'Color', color);
        set(get(get(hLine,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
    end


end
%theLegend{count+1} = 'RMSE = 0.01';

plot([min(F_List) max(F_List)], [RMSE_THRESHOLD RMSE_THRESHOLD], 'k:')
if noiseLevel>0
    %theLegend{count+2} = sprintf('Noise %4.2f%%', 100*noiseLevel);
    %plot([min(F_List) max(F_List)], [db(noiseLevel) db(noiseLevel)], 'y')
end


xlabel('Signal bandlimit / frequency f_L [Hz]', 'FontSize', fontSize);
xlabel('Sine frequency f_L [Hz]', 'FontSize', fontSize);
ylabel('RMSE (dB)', 'FontSize', fontSize);
lgd = legend(theLegend, 'FontSize', legensSize, 'Location','SouthEast');  % NorthWest
%lgd.FontName = 'Courier';

ylim([-103 -6]+shift_Y);
xlim([0.00 0.49]); % 0.03
xticks(0:0.05:0.45)
text(0.02, -15+shift_Y, plotTitle, 'FontSize', fontSize)
set(gca, 'FontSize', fontSize-2)
hold off

if iscell(plotTitle)
    plotTitle = plotTitle{1};
end

fprintf('\n---  T%02i --- %s --- --- f40 = \n', figNum, plotTitle)
for i = 1:n_methods
    fprintf('%s <- %s %s \n', osr_cell{i}, f40_cell{i}, method_cell{i})
end


function val = db(x)
    val = 20*log10(x);
end

function legTxt = OSR_text(fX)
    if ~isnan(fX)
        legTxt = sprintf('%3.2f', 0.5/fX);
    else
        legTxt = ' ---  ';
    end
end

function legTxt = f40_text(fX)
    if ~isnan(fX)
        legTxt = sprintf('%4.3f', fX);
    else
        legTxt = ' ---  ';
    end
end
